@@ -21,7 +21,7 @@
 #include "afs/param.h"
 
 RCSID
-    ("$Header: /cvs/openafs/src/afs/VNOPS/afs_vnop_remove.c,v 1.52 2006/03/13 19:18:07 rees Exp $");
+    ("$Header: /cvs/openafs/src/afs/VNOPS/afs_vnop_remove.c,v 1.52.2.3 2006/07/31 21:27:40 shadow Exp $");
 
 #include "afs/sysincludes.h"	/* Standard vendor system headers */
 #include "afsincludes.h"	/* Afs-based standard headers */
@@ -263,6 +263,9 @@ afs_remove(OSI_VC_ARG(adp), aname, acred)
 #endif
 	return code;
     }
+    if (afs_IsDynrootMount(adp)) {
+	return ENOENT;
+    }
 
     if (strlen(aname) > AFSNAMEMAX) {
 	afs_PutFakeStat(&fakestate);
