@@ -14,13 +14,14 @@
 #include "afs/param.h"
 
 RCSID
-    ("$Header: /cvs/openafs/src/afs/afs_cell.c,v 1.30.2.3 2005/09/23 14:56:26 shadow Exp $");
+    ("$Header: /cvs/openafs/src/afs/afs_cell.c,v 1.34.4.4 2007/03/20 01:13:34 shadow Exp $");
 
 #include "afs/stds.h"
 #include "afs/sysincludes.h"	/* Standard vendor system headers */
 #include "afsincludes.h"	/* Afs-based standard headers */
 #include "afs/afs_stats.h"	/* afs statistics */
 #include "afs/afs_osi.h"
+#include "afs/afs_md5.h"
 
 /* Local variables. */
 afs_rwlock_t afs_xcell;		/* Export for cmdebug peeking at locks */
@@ -132,7 +133,7 @@ afs_GetCellHostsAFSDB(char *acellName, afs_int32 * acellHosts, int *timeout,
     ObtainWriteLock(&afsdb_req.lock, 686);
 
     *acellHosts = 0;
-    afsdb_req.cellname = acellName;
+    afsdb_req.cellname = afs_strdup(acellName);
     afsdb_req.cellhosts = acellHosts;
     afsdb_req.timeout = timeout;
     afsdb_req.realname = realName;
@@ -147,6 +148,8 @@ afs_GetCellHostsAFSDB(char *acellName, afs_int32 * acellHosts, int *timeout,
 	afs_osi_Sleep(&afsdb_req);
 	ObtainReadLock(&afsdb_req.lock);
     };
+
+    afs_osi_FreeStr(afsdb_req.cellname);
     ReleaseReadLock(&afsdb_req.lock);
     ReleaseWriteLock(&afsdb_client_lock);
 
@@ -161,24 +164,41 @@ void
 afs_LookupAFSDB(char *acellName)
 {
 #ifdef AFS_AFSDB_ENV
-    afs_int32 cellHosts[MAXCELLHOSTS];
-    char *realName = NULL;
-    int code, timeout;
+    afs_int32 *cellHosts;
+    char **realName=NULL;
+    int code;
+    int *timeout=NULL;
 
-    code = afs_GetCellHostsAFSDB(acellName, cellHosts, &timeout, &realName);
+    if(!(cellHosts = afs_osi_Alloc(MAXCELLHOSTS * sizeof(afs_int32))))
+	goto done;
+
+    if(!(realName = afs_osi_Alloc(sizeof(char *))))
+	goto done;
+    *realName = NULL;
+
+    if(!(timeout = afs_osi_Alloc(sizeof(int))))
+	goto done;
+
+    code = afs_GetCellHostsAFSDB(acellName, cellHosts, timeout, realName);
     if (code)
 	goto done;
-    code = afs_NewCell(realName, cellHosts, CNoSUID, NULL, 0, 0, timeout);
+    code = afs_NewCell(*realName, cellHosts, CNoSUID, NULL, 0, 0, *timeout);
     if (code && code != EEXIST)
 	goto done;
 
     /* If we found an alias, create it */
-    if (afs_strcasecmp(acellName, realName))
-	afs_NewCellAlias(acellName, realName);
+    if (afs_strcasecmp(acellName, *realName))
+	afs_NewCellAlias(acellName, *realName);
 
   done:
-    if (realName)
-	afs_osi_FreeStr(realName);
+    if(timeout)
+	afs_osi_Free(timeout, sizeof(int));
+    if (realName && *realName)
+	afs_osi_FreeStr(*realName);
+    if(realName)
+	afs_osi_Free(realName, sizeof(char *));
+    if(cellHosts)
+	afs_osi_Free(cellHosts, MAXCELLHOSTS * sizeof(afs_int32));
 #endif
 }
 
@@ -551,6 +571,17 @@ afs_choose_cell_by_name(struct cell *cell, void *arg)
     }
 }
 
+static void *
+afs_choose_cell_by_handle(struct cell *cell, void *arg)
+{
+    if (!arg) {
+	/* Safety net */
+	return cell;
+    } else {
+	return memcmp(cell->cellHandle, (char *)arg, 16) ? NULL : cell;
+    }
+}
+
 static void *
 afs_choose_cell_by_num(struct cell *cell, void *arg)
 {
@@ -637,6 +668,17 @@ afs_GetCellByIndex(afs_int32 index, afs_int32 locktype)
     return tc;
 }
 
+struct cell *
+afs_GetCellByHandle(void *handle, afs_int32 locktype)
+{
+    struct cell *tc;
+
+    tc = afs_TraverseCells(&afs_choose_cell_by_handle, handle);
+    if (tc)
+	afs_UpdateCellLRU(tc);
+    return tc;
+}
+
 struct cell *
 afs_GetPrimaryCell(afs_int32 locktype)
 {
@@ -706,10 +748,10 @@ afs_NewCell(char *acellName, afs_int32 * acellHosts, int aflags,
 	tc->cellName = afs_strdup(acellName);
 	tc->fsport = AFS_FSPORT;
 	tc->vlport = AFS_VLPORT;
+	AFS_MD5_String(tc->cellHandle, tc->cellName, strlen(tc->cellName));
 	RWLOCK_INIT(&tc->lock, "cell lock");
 	newc = 1;
-	if (afs_thiscell && !strcmp(acellName, afs_thiscell))
-	    aflags &= ~CNoSUID;
+	aflags |= CNoSUID;
     }
     ObtainWriteLock(&tc->lock, 688);
 
