@@ -23,7 +23,7 @@
 #include "afs/param.h"
 
 RCSID
-    ("$Header: /cvs/openafs/src/afs/afs_osi_pag.c,v 1.29.4.6 2006/12/20 20:08:32 shadow Exp $");
+    ("$Header: /cvs/openafs/src/afs/afs_osi_pag.c,v 1.29.4.9 2007/02/09 00:27:00 shadow Exp $");
 
 #include "afs/sysincludes.h"	/* Standard vendor system headers */
 #include "afsincludes.h"	/* Afs-based standard headers */
@@ -44,6 +44,11 @@ afs_uint32 pagCounter = 1;
 afs_uint32 pagCounter = 0;
 #endif /* UKERNEL && AFS_WEB_ENHANCEMENTS */
 
+#ifdef AFS_LINUX26_ONEGROUP_ENV
+#define NUMPAGGROUPS 1
+#else
+#define NUMPAGGROUPS 2
+#endif
 /* Local variables */
 
 /*
@@ -363,6 +368,7 @@ afs_setpag_val(int pagval)
     return (code);
 }
 
+#ifndef AFS_LINUX26_ONEGROUP_ENV
 int
 afs_getpag_val()
 {
@@ -376,13 +382,13 @@ afs_getpag_val()
     gidset0 = gids[0];
     gidset1 = gids[1];
 #else
-
     gidset0 = credp->cr_groups[0];
     gidset1 = credp->cr_groups[1];
 #endif
     pagvalue = afs_get_pag_from_groups(gidset0, gidset1);
     return pagvalue;
 }
+#endif
 #endif /* UKERNEL && AFS_WEB_ENHANCEMENTS */
 
 
@@ -423,8 +429,10 @@ afs_InitReq(register struct vrequest *av, struct AFS_UCRED *acred)
 	return EIO;
 
 #ifdef AFS_LINUX26_ENV
+#if !defined(AFS_NONFSTRANS)
     if (osi_linux_nfs_initreq(av, acred, &code))
 	return code;
+#endif
 #endif
 
     av->uid = PagInCred(acred);
@@ -450,7 +458,22 @@ afs_InitReq(register struct vrequest *av, struct AFS_UCRED *acred)
 }
 
 
+#ifdef AFS_LINUX26_ONEGROUP_ENV
+afs_uint32
+afs_get_pag_from_groups(struct group_info *group_info)
+{
+    afs_uint32 g0 = 0;
+    afs_uint32 i;
 
+    AFS_STATCNT(afs_get_pag_from_groups);
+    for (i = 0; (i < group_info->ngroups && 
+		 (g0 = GROUP_AT(group_info, i)) != (gid_t) NOGROUP); i++) {
+	if (((g0 >> 24) & 0xff) == 'A')
+	    return g0;
+    }
+    return NOPAG;
+}
+#else
 afs_uint32
 afs_get_pag_from_groups(gid_t g0a, gid_t g1a)
 {
@@ -459,6 +482,7 @@ afs_get_pag_from_groups(gid_t g0a, gid_t g1a)
     afs_uint32 h, l, ret;
 
     AFS_STATCNT(afs_get_pag_from_groups);
+
     g0 -= 0x3f00;
     g1 -= 0x3f00;
     if (g0 < 0xc000 && g1 < 0xc000) {
@@ -476,7 +500,7 @@ afs_get_pag_from_groups(gid_t g0a, gid_t g1a)
     }
     return NOPAG;
 }
-
+#endif
 
 void
 afs_get_groups_from_pag(afs_uint32 pag, gid_t * g0p, gid_t * g1p)
@@ -485,6 +509,10 @@ afs_get_groups_from_pag(afs_uint32 pag, gid_t * g0p, gid_t * g1p)
 
 
     AFS_STATCNT(afs_get_groups_from_pag);
+#ifdef AFS_LINUX26_ONEGROUP_ENV
+    *g0p = pag;
+    *g1p = 0;
+#else
 #if !defined(UKERNEL) || !defined(AFS_WEB_ENHANCEMENTS)
     pag &= 0x7fffffff;
 #endif /* UKERNEL && AFS_WEB_ENHANCEMENTS */
@@ -494,6 +522,7 @@ afs_get_groups_from_pag(afs_uint32 pag, gid_t * g0p, gid_t * g1p)
     g1 |= ((pag >> 28) % 3) << 14;
     *g0p = g0 + 0x3f00;
     *g1p = g1 + 0x3f00;
+#endif
 }
 
 
@@ -534,7 +563,7 @@ PagInCred(const struct AFS_UCRED *cred)
 	return NOPAG;
     }
 #elif defined(AFS_LINUX26_ENV)
-    if (cred->cr_group_info->ngroups < 2) {
+    if (cred->cr_group_info->ngroups < NUMPAGGROUPS) {
 	pag = NOPAG;
 	goto out;
     }
@@ -551,6 +580,7 @@ PagInCred(const struct AFS_UCRED *cred)
 #if defined(AFS_AIX51_ENV)
     g0 = cred->cr_groupset.gs_union.un_groups[0];
     g1 = cred->cr_groupset.gs_union.un_groups[1];
+#elif defined(AFS_LINUX26_ONEGROUP_ENV)
 #elif defined(AFS_LINUX26_ENV)
     g0 = GROUP_AT(cred->cr_group_info, 0);
     g1 = GROUP_AT(cred->cr_group_info, 1);
@@ -562,20 +592,25 @@ PagInCred(const struct AFS_UCRED *cred)
     g1 = cred->cr_groups[1];
 #endif
 #endif
+#if defined(AFS_LINUX26_ONEGROUP_ENV)
+    pag = (afs_int32) afs_get_pag_from_groups(cred->cr_group_info);
+#else
     pag = (afs_int32) afs_get_pag_from_groups(g0, g1);
+#endif
 out:
 #if defined(AFS_LINUX26_ENV) && defined(LINUX_KEYRING_SUPPORT)
     if (pag == NOPAG) {
 	struct key *key;
-	afs_uint32 pag, newpag;
+	afs_uint32 upag, newpag;
 
 	key = request_key(&key_type_afs_pag, "_pag", NULL);
 	if (!IS_ERR(key)) {
 	    if (key_validate(key) == 0 && key->uid == 0) {	/* also verify in the session keyring? */
-
-		pag = (afs_uint32) key->payload.value;
-		if (((pag >> 24) & 0xff) == 'A')
-		    __setpag(&cred, pag, &newpag, 0);
+		upag = (afs_uint32) key->payload.value;
+		if (((upag >> 24) & 0xff) == 'A') {
+		    __setpag(&cred, upag, &newpag, 0);
+		    pag = (afs_int32) upag;
+		}
 	    }
 	    key_put(key);
 	} 
