@@ -24,7 +24,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/ptserver/ptutils.c,v 1.15 2004/04/18 06:13:50 kolya Exp $");
+    ("$Header: /cvs/openafs/src/ptserver/ptutils.c,v 1.19.6.1 2007/04/10 18:39:54 shadow Exp $");
 
 #include <afs/stds.h>
 #include <sys/types.h>
@@ -53,7 +53,7 @@ RCSID
 /* Foreign cells are represented by the group system:authuser@cell*/
 #define AUTHUSER_GROUP "system:authuser"
 
-
+extern int restricted;
 extern struct ubik_dbase *dbase;
 extern struct afsconf_dir *prdir;
 extern int pr_noAuth;
@@ -62,6 +62,9 @@ extern int IDCmp();
 extern afs_int32 AddToEntry();
 static char *whoami = "ptserver";
 
+int prp_user_default = PRP_USER_DEFAULT;
+int prp_group_default = PRP_GROUP_DEFAULT;
+
 #if defined(SUPERGROUPS)
 
 #include "map.h"
@@ -91,10 +94,7 @@ struct map *sg_found;
 int (*pt_save_dbase_write) ();
 
 int
-pt_mywrite(tdb, fno, bp, pos, count)
-     struct ubik_dbase *tdb;
-     afs_int32 fno, pos, count;
-     char *bp;
+pt_mywrite(struct ubik_dbase *tdb, afs_int32 fno, char *bp, afs_int32 pos, afs_int32 count)
 {
     afs_uint32 headersize = ntohl(cheader.headerSize);
 
@@ -188,6 +188,7 @@ pt_mywrite(tdb, fno, bp, pos, count)
  *  just after ubik_ServerInit.
  */
 
+void 
 pt_hook_write()
 {
     extern struct ubik_dbase *ubik_dbase;
@@ -208,8 +209,7 @@ pt_hook_write()
  *   return one if name is OK and zero if name is bogus. */
 
 static int
-CorrectUserName(name)
-     char *name;
+CorrectUserName(char *name)
 {
     extern int pr_realmNameLen;
 
@@ -227,12 +227,10 @@ CorrectUserName(name)
  * rename, which then compares the correct name with the requested new name. */
 
 static afs_int32
-CorrectGroupName(ut, aname, cid, oid, cname)
-     struct ubik_trans *ut;
-     char aname[PR_MAXNAMELEN];	/* name for group */
-     afs_int32 cid;		/* caller id */
-     afs_int32 oid;		/* owner of group */
-     char cname[PR_MAXNAMELEN];	/* correct name for group */
+CorrectGroupName(struct ubik_trans *ut, char aname[PR_MAXNAMELEN],	/* name for group */
+		 afs_int32 cid,		/* caller id */
+		 afs_int32 oid,		/* owner of group */
+		 char cname[PR_MAXNAMELEN])	/* correct name for group */
 {
     afs_int32 code;
     int admin;
@@ -308,12 +306,10 @@ CorrectGroupName(ut, aname, cid, oid, cname)
 }
 
 int
-AccessOK(ut, cid, tentry, mem, any)
-     struct ubik_trans *ut;
-     afs_int32 cid;		/* caller id */
-     struct prentry *tentry;	/* object being accessed */
-     int mem;			/* check membership in aid, if group */
-     int any;			/* if set return true */
+AccessOK(struct ubik_trans *ut, afs_int32 cid,		/* caller id */
+	 struct prentry *tentry,	/* object being accessed */
+	 int mem,			/* check membership in aid, if group */
+	 int any)			/* if set return true */
 {
     afs_int32 flags;
     afs_int32 oid;
@@ -323,6 +319,8 @@ AccessOK(ut, cid, tentry, mem, any)
 	return 1;
     if (cid == SYSADMINID)
 	return 1;		/* special case fileserver */
+    if (restricted && ((mem == PRP_ADD_MEM) || (mem == any == 0)))
+	return 0;
     if (tentry) {
 	flags = tentry->flags;
 	oid = tentry->owner;
@@ -332,9 +330,9 @@ AccessOK(ut, cid, tentry, mem, any)
     }
     if (!(flags & PRACCESS)) {	/* provide default access */
 	if (flags & PRGRP)
-	    flags |= PRP_GROUP_DEFAULT;
+	    flags |= prp_group_default;
 	else
-	    flags |= PRP_USER_DEFAULT;
+	    flags |= prp_user_default;
     }
 
     if (flags & any)
@@ -360,14 +358,7 @@ AccessOK(ut, cid, tentry, mem, any)
 }
 
 afs_int32
-CreateEntry(at, aname, aid, idflag, flag, oid, creator)
-     struct ubik_trans *at;
-     char aname[PR_MAXNAMELEN];
-     afs_int32 *aid;
-     afs_int32 idflag;
-     afs_int32 flag;
-     afs_int32 oid;
-     afs_int32 creator;
+CreateEntry(struct ubik_trans *at, char aname[PR_MAXNAMELEN], afs_int32 *aid, afs_int32 idflag, afs_int32 flag, afs_int32 oid, afs_int32 creator)
 {
     /* get and init a new entry */
     afs_int32 code;
@@ -633,10 +624,7 @@ CreateEntry(at, aname, aid, idflag, flag, oid, creator)
  * entry if appropriate */
 
 afs_int32
-RemoveFromEntry(at, aid, bid)
-     struct ubik_trans *at;
-     afs_int32 aid;
-     afs_int32 bid;
+RemoveFromEntry(struct ubik_trans *at, afs_int32 aid, afs_int32 bid)
 {
     afs_int32 code;
     struct prentry tentry;
@@ -724,11 +712,7 @@ RemoveFromEntry(at, aid, bid)
  * entry if appropriate */
 
 afs_int32
-ChangeIDEntry(at, aid, newid, bid)
-     register struct ubik_trans *at;
-     register afs_int32 aid;
-     register afs_int32 bid;
-     afs_int32 newid;
+ChangeIDEntry(register struct ubik_trans *at, register afs_int32 aid, afs_int32 newid, register afs_int32 bid)
 {
     register afs_int32 code;
     struct prentry tentry;
@@ -796,10 +780,7 @@ ChangeIDEntry(at, aid, newid, bid)
  * continuation entry if appropriate */
 
 afs_int32
-RemoveFromSGEntry(at, aid, bid)
-     register struct ubik_trans *at;
-     register afs_int32 aid;
-     register afs_int32 bid;
+RemoveFromSGEntry(register struct ubik_trans *at, register afs_int32 aid, register afs_int32 bid)
 {
     register afs_int32 code;
     struct prentry tentry;
@@ -897,10 +878,7 @@ RemoveFromSGEntry(at, aid, bid)
  * groups, putting groups owned by it on orphan chain, and freeing the space */
 
 afs_int32
-DeleteEntry(at, tentry, loc)
-     struct ubik_trans *at;
-     struct prentry *tentry;
-     afs_int32 loc;
+DeleteEntry(struct ubik_trans *at, struct prentry *tentry, afs_int32 loc)
 {
     afs_int32 code;
     struct contentry centry;
@@ -1051,11 +1029,7 @@ DeleteEntry(at, tentry, loc)
  * Note the entry is written out by this routine. */
 
 afs_int32
-AddToEntry(tt, entry, loc, aid)
-     struct ubik_trans *tt;
-     struct prentry *entry;
-     afs_int32 loc;
-     afs_int32 aid;
+AddToEntry(struct ubik_trans *tt, struct prentry *entry, afs_int32 loc, afs_int32 aid)
 {
     afs_int32 code;
     afs_int32 i;
@@ -1168,11 +1142,7 @@ AddToEntry(tt, entry, loc, aid)
  * Note the entry is written out by this routine. */
 
 afs_int32
-AddToSGEntry(tt, entry, loc, aid)
-     struct ubik_trans *tt;
-     struct prentry *entry;
-     afs_int32 loc;
-     afs_int32 aid;
+AddToSGEntry(struct ubik_trans *tt, struct prentry *entry, afs_int32 loc, afs_int32 aid)
 {
     register afs_int32 code;
     afs_int32 i;
@@ -1281,10 +1251,7 @@ AddToSGEntry(tt, entry, loc, aid)
 #endif /* SUPERGROUPS */
 
 afs_int32
-AddToPRList(alist, sizeP, id)
-     prlist *alist;
-     int *sizeP;
-     afs_int32 id;
+AddToPRList(prlist *alist, int *sizeP, afs_int32 id)
 {
     char *tmp;
     int count;
@@ -1307,11 +1274,7 @@ AddToPRList(alist, sizeP, id)
 }
 
 afs_int32
-GetList(at, tentry, alist, add)
-     struct ubik_trans *at;
-     struct prentry *tentry;
-     prlist *alist;
-     afs_int32 add;
+GetList(struct ubik_trans *at, struct prentry *tentry, prlist *alist, afs_int32 add)
 {
     afs_int32 code;
     afs_int32 i;
@@ -1386,12 +1349,7 @@ GetList(at, tentry, alist, add)
 
 
 afs_int32
-GetList2(at, tentry, tentry2, alist, add)
-     struct ubik_trans *at;
-     struct prentry *tentry;
-     struct prentry *tentry2;
-     prlist *alist;
-     afs_int32 add;
+GetList2(struct ubik_trans *at, struct prentry *tentry, struct prentry *tentry2, prlist *alist, afs_int32 add)
 {
     afs_int32 code = 0;
     afs_int32 i;
@@ -1499,12 +1457,7 @@ GetList2(at, tentry, tentry2, alist, add)
 #if defined(SUPERGROUPS)
 
 afs_int32
-GetListSG2(at, gid, alist, sizeP, depth)
-     struct ubik_trans *at;
-     afs_int32 gid;
-     prlist *alist;
-     afs_int32 depth;
-     afs_int32 *sizeP;
+GetListSG2(struct ubik_trans *at, afs_int32 gid, prlist *alist, afs_int32 *sizeP, afs_int32 depth)
 {
     register afs_int32 code;
     struct prentry tentry;
@@ -1620,10 +1573,7 @@ GetListSG2(at, gid, alist, sizeP, depth)
 }
 
 afs_int32
-GetSGList(at, tentry, alist)
-     struct ubik_trans *at;
-     struct prentry *tentry;
-     prlist *alist;
+GetSGList(struct ubik_trans *at, struct prentry *tentry, prlist *alist)
 {
     register afs_int32 code;
     afs_int32 i;
@@ -1677,10 +1627,7 @@ GetSGList(at, tentry, alist)
 #endif /* SUPERGROUPS */
 
 afs_int32
-GetOwnedChain(ut, next, alist)
-     struct ubik_trans *ut;
-     afs_int32 *next;
-     prlist *alist;
+GetOwnedChain(struct ubik_trans *ut, afs_int32 *next, prlist *alist)
 {
     afs_int32 code;
     struct prentry tentry;
@@ -1711,10 +1658,7 @@ GetOwnedChain(ut, next, alist)
 }
 
 afs_int32
-GetMax(at, uid, gid)
-     struct ubik_trans *at;
-     afs_int32 *uid;
-     afs_int32 *gid;
+GetMax(struct ubik_trans *at, afs_int32 *uid, afs_int32 *gid)
 {
     *uid = ntohl(cheader.maxID);
     *gid = ntohl(cheader.maxGroup);
@@ -1722,10 +1666,7 @@ GetMax(at, uid, gid)
 }
 
 afs_int32
-SetMax(at, id, flag)
-     struct ubik_trans *at;
-     afs_int32 id;
-     afs_int32 flag;
+SetMax(struct ubik_trans *at, afs_int32 id, afs_int32 flag)
 {
     afs_int32 code;
     if (flag & PRGRP) {
@@ -1747,8 +1688,7 @@ SetMax(at, id, flag)
 }
 
 afs_int32
-read_DbHeader(tt)
-     struct ubik_trans *tt;
+read_DbHeader(struct ubik_trans *tt)
 {
     afs_int32 code;
 
@@ -1757,7 +1697,7 @@ read_DbHeader(tt)
 
     code = pr_Read(tt, 0, 0, (char *)&cheader, sizeof(cheader));
     if (code != 0) {
-	com_err(whoami, code, "Couldn't read header");
+	afs_com_err(whoami, code, "Couldn't read header");
     }
     return code;
 }
@@ -1795,7 +1735,7 @@ Initdb()
     len = sizeof(cheader);
     code = pr_Read(tt, 0, 0, (char *)&cheader, len);
     if (code != 0) {
-	com_err(whoami, code, "couldn't read header");
+	afs_com_err(whoami, code, "couldn't read header");
 	ubik_AbortTrans(tt);
 	return code;
     }
@@ -1822,14 +1762,14 @@ Initdb()
 	for (i = 0; i < sizeof(cheader); i++)
 	    if (bp[i]) {
 		code = PRDBBAD;
-		com_err(whoami, code,
+		afs_com_err(whoami, code,
 			"Can't rebuild database because it is not empty");
 		return code;
 	    }
     }
     if (!pr_noAuth) {
 	code = PRDBBAD;
-	com_err(whoami, code,
+	afs_com_err(whoami, code,
 		"Can't rebuild database because not running NoAuth");
 	return code;
     }
@@ -1864,7 +1804,7 @@ Initdb()
     if ((code = set_header_word(tt, version, htonl(PRDBVERSION)))
 	|| (code = set_header_word(tt, headerSize, htonl(sizeof(cheader))))
 	|| (code = set_header_word(tt, eofPtr, cheader.headerSize))) {
-	com_err(whoami, code, "couldn't write header words");
+	afs_com_err(whoami, code, "couldn't write header words");
 	ubik_AbortTrans(tt);
 	return code;
     }
@@ -1874,7 +1814,7 @@ Initdb()
     code = CreateEntry		      \
 	(tt, (name), &temp, /*idflag*/1, flag, SYSADMINID, SYSADMINID); \
     if (code) {			      \
-	com_err (whoami, code, "couldn't create %s with id %di.", 	\
+	afs_com_err (whoami, code, "couldn't create %s with id %di.", 	\
 		 (name), (id));	      \
 	ubik_AbortTrans(tt);	      \
 	return code;		      \
@@ -1892,7 +1832,7 @@ Initdb()
      * it back to 0 */
     code = set_header_word(tt, maxID, 0);	/* correct in any byte order */
     if (code) {
-	com_err(whoami, code, "couldn't reset max id");
+	afs_com_err(whoami, code, "couldn't reset max id");
 	ubik_AbortTrans(tt);
 	return code;
     }
@@ -1904,13 +1844,7 @@ Initdb()
 }
 
 afs_int32
-ChangeEntry(at, aid, cid, name, oid, newid)
-     struct ubik_trans *at;
-     afs_int32 aid;
-     afs_int32 cid;
-     char *name;
-     afs_int32 oid;
-     afs_int32 newid;
+ChangeEntry(struct ubik_trans *at, afs_int32 aid, afs_int32 cid, char *name, afs_int32 oid, afs_int32 newid)
 {
     afs_int32 code;
     afs_int32 i, nptr, pos;
@@ -1934,6 +1868,8 @@ ChangeEntry(at, aid, cid, name, oid, newid)
     code = pr_ReadEntry(at, 0, loc, &tentry);
     if (code)
 	return PRDBFAIL;
+    if (restricted && !IsAMemberOf(at, cid, SYSADMINID)) 
+	return PRPERM;
     if (tentry.owner != cid && !IsAMemberOf(at, cid, SYSADMINID)
 	&& !IsAMemberOf(at, cid, tentry.owner) && !pr_noAuth)
 	return PRPERM;
@@ -2243,10 +2179,8 @@ inRange(struct prentry *cellEntry, afs_int32 aid)
 
 }
 
-AddAuthGroup(tentry, alist, size)
-     struct prentry *tentry;
-     prlist *alist;
-     afs_int32 *size;
+int
+AddAuthGroup(struct prentry *tentry, prlist *alist, afs_int32 *size)
 {
     if (!(strchr(tentry->name, '@')))
 	return (AddToPRList(alist, size, AUTHUSERID));
