@@ -49,6 +49,8 @@
 #include <afs/param.h>
 #endif
 
+RCSID
+    ("$Header: /cvs/openafs/src/util/uuid.c,v 1.17 2005/12/24 01:07:51 shadow Exp $");
 
 #ifdef KERNEL
 #include "afs/sysincludes.h"
@@ -58,7 +60,6 @@
 #else /* KERNEL */
 #include <stdio.h>
 #include <errno.h>
-#include <string.h>
 #ifdef AFS_NT40_ENV
 #include <winsock2.h>
 #include <process.h>
@@ -72,6 +73,13 @@
 #include <sys/time.h>
 #endif /* ITIMER_REAL */
 #include <net/if.h>
+#ifdef HAVE_STRING_H
+#include <string.h>
+#else
+#ifdef HAVE_STRINGS_H
+#include <strings.h>
+#endif
+#endif
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
